//MIDI LIBRARY
#include <MIDI.h>
//CONSTRUCTOR FOR CONTROLS
#include "Controller.h"

//Define LEDS
#define LED_FLTA 22            //Filter Deck 1 On/Off
#define LED_FLTB 24            //Filter Deck 2 On/Off
#define LED_FXA 26             //FX 1 On/Off
#define LED_FXB 28             //FX 2 On/Off

#define LED_FXDA1 30           //FX 1 Deck 1 On/Off
#define LED_FXDA2 32          //FX 1 Deck 2 On/Off
#define LED_FXDB1 34          //FX 2 Deck 1 On/Off
#define LED_FXDB2 36          //FX 2 Deck 2 On/Off

#define LED_R 42           //Switch state RGB
#define LED_G 44
#define LED_B 46


//CONSTANTS
byte NUMBER_BUTTONS = 10;
byte NUMBER_POTS = 8;
bool switched = true;

// Need to create a custom INSTANCE for use with different BAUD Rates
struct HairlessMidiSettings : public midi::DefaultSettings
{
   static const bool UseRunningStatus = false;
   static const long BaudRate = 9600;
};

//Create a custom instance with custom settings
MIDI_CREATE_CUSTOM_INSTANCE(HardwareSerial, Serial, MIDI, HairlessMidiSettings);

//Button (Pin Number, Command, Note Number, Channel, Debounce Time, Mutable)
//** Command parameter 0=NOTE  1=CC  2=Toggle CC ** 3=Special switch
//10 buttons
Button BO1(7, 2, 3, 1, 1, false);     //Filter A On/Off
Button BO2(6, 2, 5, 1, 1, false);     //Filter B On/Off
Button BO3(5, 2, 7, 1, 1, false);     //FX 1 On/Off
Button BO4(4, 2, 9, 1, 1, false);     //FX 2 On/Off
Button BO5(8, 2, 14, 1, 1, true);     //FX Button 1
Button BO6(9, 2, 15, 1, 1, true);     //FX Button 2
Button BO7(10, 2, 16, 1, 1, true);     //FX Button 3
Button BO8(11, 2, 22, 1, 1, true);    //FX 1/2 - Deck 1 On/Off
Button BO9(12, 2, 23, 1, 1, true);    //FX 1/2 - Deck 2 On/Off
Button BO10(13, 3, 100, 1, 1, false);  //-- Special switch

// Full list of buttons to update
Button *BUTTONS[] {&BO1, &BO2, &BO3, &BO4, &BO5, &BO6, &BO7, &BO8, &BO9, &BO10};
  
// Pot <name>(<pin_number>, <control>, <channel_number>, <pot_sensitivity>, <can_mutate?>);
Pot PO1(A0, 2, 1, 3, false);        // Filter A 
Pot PO2(A1, 4, 1, 3, false);        // FX 1         POR CORREGIR
Pot PO3(A2, 6, 1, 3, false);        // Filter B     POR CORREGIR
Pot PO4(A3, 8, 1, 3, false);        // FX 2
Pot PO5(A4, 10, 1, 3, true);        // FX Selector 
Pot PO6(A5, 11, 1, 3, true);        // FX Param 1
Pot PO7(A6, 12, 1, 3, true);        // FX Param 2
Pot PO8(A7, 13, 1, 3, true);        // FX Param 3

// Full list of pots to update
Pot *POTS[] {&PO1, &PO2, &PO3, &PO4, &PO5, &PO6, &PO7, &PO8};

// --------- SETUP ---------
void setup() 
{
  //LEDS SETUP
  pinMode(LED_FLTA, OUTPUT);
  pinMode(LED_FLTB, OUTPUT);
  pinMode(LED_FXA, OUTPUT);
  pinMode(LED_FXB, OUTPUT);

  pinMode(LED_FXDA1, OUTPUT);
  pinMode(LED_FXDA2, OUTPUT);
  pinMode(LED_FXDB1, OUTPUT);
  pinMode(LED_FXDB2, OUTPUT);

  pinMode(LED_R, OUTPUT);
  pinMode(LED_G, OUTPUT);
  pinMode(LED_B, OUTPUT);
  //initial config
  digitalWrite(LED_FLTA, LOW);
  digitalWrite(LED_FLTB, LOW);
  digitalWrite(LED_FXA, LOW);
  digitalWrite(LED_FXB, LOW);
  digitalWrite(LED_FXDA1, LOW);
  digitalWrite(LED_FXDA2, LOW);
  digitalWrite(LED_FXDB1, LOW);
  digitalWrite(LED_FXDB2, LOW);
  //pink
  digitalWrite(LED_R, HIGH);
  digitalWrite(LED_G, LOW);
  digitalWrite(LED_B, HIGH);

  //set custom handlers
  MIDI.setHandleNoteOn(MyHandleNoteOn);
  MIDI.setHandleNoteOff(MyHandleNoteOff);
  MIDI.setHandleControlChange(MyControlChange);
  MIDI.setHandleProgramChange(MyProgramChange);
  MIDI.begin(MIDI_CHANNEL_OMNI);  // Listen to all incoming messages
  //this is very important to not get errors
  MIDI.turnThruOff();
  //Serial.begin(9600);
  delay(5000);
}

// CUSTOM HANDLERS
// NOTE ON
void MyHandleNoteOn(byte channel, byte note, byte velocity) 
{

}

//NOTE OFF
void MyHandleNoteOff(byte channel, byte note, byte velocity) 
{

}

void MyProgramChange(byte channel, byte number)
{

}

void MyControlChange(byte channel, byte number, byte value)
{

}

// --------- LOOP ---------
void loop() 
{
  // Read incoming messages -- NECCESARY ?
  MIDI.read();
  // Send
  updateButtons();
  updatePots();
  // Switch RGB LED
  if (switched) {
    digitalWrite(LED_R, LOW);
    digitalWrite(LED_G, LOW);
    digitalWrite(LED_B, HIGH);
  }
  else {
    digitalWrite(LED_R, LOW);
    digitalWrite(LED_G, HIGH);
    digitalWrite(LED_B, LOW);
  }
  //IS THIS NECESSARY ? ...  YES
  if (MIDI.read())                // Is there a MIDI message incoming ?
    {
      switch(MIDI.getData1())      // Get the type of the message we caught
        {
            case 26:
              digitalWrite(LED_FLTA, MIDI.getData2());
              break;
            case 27:
              digitalWrite(LED_FLTB, MIDI.getData2());
              break;
            case 28:
              digitalWrite(LED_FXA, MIDI.getData2());
              break;
            case 29:
              digitalWrite(LED_FXB, MIDI.getData2());
              break;
            case 30:
              digitalWrite(LED_FXDA1, MIDI.getData2());
              break;
            case 31:
              digitalWrite(LED_FXDA2, MIDI.getData2());
              break;
            case 32:
              digitalWrite(LED_FXDB1, MIDI.getData2());
              break;
            case 33:
              digitalWrite(LED_FXDB2, MIDI.getData2());
              break;
            // See the online reference for other message types
            default:
                break;
        }
    }
  
}

void updateButtons()
{
  // Cycle through Button array
  for (int i = 0; i < NUMBER_BUTTONS; i = i + 1) {
    byte message = BUTTONS[i]->getValue();

    //  Button is pressed
    if (message == 0) {
      switch (BUTTONS[i]->Bcommand) {
        case 0: //Note
          //sendMidi(144, BUTTONS[i]->Bvalue, 127);
          MIDI.sendNoteOn(BUTTONS[i]->Bvalue, 127, BUTTONS[i]->Bchannel);
          break;
        case 1: //CC
          //sendMidi(176, BUTTONS[i]->Bvalue, 127);
          MIDI.sendControlChange(BUTTONS[i]->Bvalue, 127, BUTTONS[i]->Bchannel);
          break;
        case 2: //Toggle
          if (BUTTONS[i]->Btoggle == 0) {
            //mutable
            if (BUTTONS[i]->mutante) {
              if (switched){
                MIDI.sendControlChange(BUTTONS[i]->Bvalue, 127, 2);
              }
              else {
                MIDI.sendControlChange(BUTTONS[i]->Bvalue, 127, 1);
              }
            }
            else {
              MIDI.sendControlChange(BUTTONS[i]->Bvalue, 127, BUTTONS[i]->Bchannel);
            }
            
            BUTTONS[i]->Btoggle = 1;
          }
          else if (BUTTONS[i]->Btoggle == 1) {
            if (BUTTONS[i]->mutante) {
              if (switched){
                MIDI.sendControlChange(BUTTONS[i]->Bvalue, 0, 2);
              }
              else {
                MIDI.sendControlChange(BUTTONS[i]->Bvalue, 0, 1);
              }
            }
            else {
              MIDI.sendControlChange(BUTTONS[i]->Bvalue, 0, BUTTONS[i]->Bchannel);
            }
            
            BUTTONS[i]->Btoggle = 0;
          }
          break;
          case 3: //New Type, Switch
          switched = !switched;
          break;
      }
    }
  
    //  Button is not pressed
    if (message == 1) {
      switch (BUTTONS[i]->Bcommand) {
        case 0:
          //sendMidi(128, BUTTONS[i]->Bvalue, 0);
          MIDI.sendNoteOff(BUTTONS[i]->Bvalue, 0, BUTTONS[i]->Bchannel);
          break;
        case 1:
          //sendMidi(176, BUTTONS[i]->Bvalue, 0);
          MIDI.sendControlChange(BUTTONS[i]->Bvalue, 0, BUTTONS[i]->Bchannel);
          break;
        default:
        break;
      }
    }
    
  }
}

void updatePots(){
  //loop for number of potentiometers
  for (int i = 0; i < NUMBER_POTS; i = i + 1) {
    //update first
    POTS[i]->updatePot();
    //check for action
    if(POTS[i]->checkPot())
    {
      //check if the pot can mutate
      if(POTS[i]->mutante)
      {
        //check the state of the switch
        if (switched)
        {
          MIDI.sendControlChange(POTS[i]->potControl, POTS[i]->potValue, 2);
        }
        else 
        {
          MIDI.sendControlChange(POTS[i]->potControl, POTS[i]->potValue, 1);
        }
      }
      else 
      {
        MIDI.sendControlChange(POTS[i]->potControl, POTS[i]->potValue, POTS[i]->potChannel);
      }
    }
  }
}

